// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

package frc.robot;

import edu.wpi.first.wpilibj.GenericHID;
import edu.wpi.first.wpilibj.Joystick;
import edu.wpi.first.wpilibj.PowerDistribution;
import edu.wpi.first.wpilibj.XboxController;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.button.JoystickButton;
import frc.robot.commands.Auto;
import frc.robot.commands.Blow;
import frc.robot.commands.DriveWithJoysticks;
import frc.robot.commands.SetShear;
import frc.robot.commands.SetShooterSpeed;
import frc.robot.commands.ShearGoBrr;
import frc.robot.commands.Shoot;
import frc.robot.commands.Succ;
import frc.robot.commands.ZeroGyro;
import frc.robot.subsystems.Chassis;
import frc.robot.subsystems.Gatherer;
import frc.robot.subsystems.PotatoCannon;
import frc.robot.subsystems.Shear;

/**
 * This class is where the bulk of the robot should be declared. Since Command-based is a
 * "declarative" paradigm, very little robot logic should actually be handled in the {@link Robot}
 * periodic methods (other than the scheduler calls). Instead, the structure of the robot (including
 * subsystems, commands, and button mappings) should be declared here.
 */
public class RobotContainer {
  // The robot's subsystems and commands are defined here...
  public static PowerDistribution s_pdp = new PowerDistribution();

  public static Joystick s_driver = new Joystick(0);
  public static Joystick s_manip = new Joystick(1);

  public final Chassis m_chassis = new Chassis();
  public final PotatoCannon m_potatoCannon = new PotatoCannon();
  public final Gatherer m_gatherer = new Gatherer();
  public final Shear m_shear = new Shear();

  private final Auto m_autoCommand = new Auto(m_chassis, m_potatoCannon, m_gatherer);

  /** The container for the robot. Contains subsystems, OI devices, and commands. */
  public RobotContainer() {
    // Configure the button bindings
    configureButtonBindings();

    m_chassis.setDefaultCommand(new DriveWithJoysticks(m_chassis));
  }

  /**
   * Use this method to define your button->command mappings. Buttons can be created by
   * instantiating a {@link GenericHID} or one of its subclasses ({@link
   * edu.wpi.first.wpilibj.Joystick} or {@link XboxController}), and then passing it to a {@link
   * edu.wpi.first.wpilibj2.command.button.JoystickButton}.
   */
  private void configureButtonBindings() {
    new JoystickButton(s_manip, 4).whenPressed(new SetShooterSpeed(m_potatoCannon, Constants.SHOOTER_RPM));
    new JoystickButton(s_manip, 3).whenPressed(new SetShooterSpeed(m_potatoCannon, Constants.SHOOTER_RPM/3.0*2.0));
    new JoystickButton(s_manip, 1).whenPressed(new SetShooterSpeed(m_potatoCannon, 0.4));
    new JoystickButton(s_manip, 2).whenPressed(new SetShooterSpeed(m_potatoCannon, 0.0));

    new JoystickButton(s_driver, 6).whileHeld(new Succ(m_gatherer));
    new JoystickButton(s_driver, 8).whileHeld(new Blow(m_gatherer));

    new JoystickButton(s_driver, 10).whenPressed(new ZeroGyro(m_chassis));

    new JoystickButton(s_manip, 5).whileHeld(new Shoot(m_potatoCannon, m_gatherer));

    new JoystickButton(s_manip, 9).whenPressed(new SetShear(m_shear, 90.0));

    // new JoystickButton(s_manip, 10).whenPressed(new SequentialCommandGroup(
    //   new SetShear(m_shear, 90.0),
    //   new SetShear(m_shear, 45.0),
    //   new SetShear(m_shear, -45.0)
    // ));
    new JoystickButton(s_manip, 10).whileHeld(new ShearGoBrr(m_shear));
  }

  /**
   * Use this to pass the autonomous command to the main {@link Robot} class.
   *
   * @return the command to run in autonomous
   */
  public Command getAutonomousCommand() {
    // An ExampleCommand will run in autonomous
    return m_autoCommand;
  }
}
