package frc.robot;

import com.ctre.phoenix.motorcontrol.ControlMode;
import com.ctre.phoenix.motorcontrol.FeedbackDevice;
import com.ctre.phoenix.motorcontrol.SupplyCurrentLimitConfiguration;
import com.ctre.phoenix.motorcontrol.can.TalonFX;
import com.ctre.phoenix.motorcontrol.can.TalonSRX;

public class SwervePod {
  public TalonSRX turn;
  public TalonFX drive;

  public SwervePod(int turn_id, int drive_id) {
      turn = new TalonSRX(turn_id);
      drive = new TalonFX(drive_id);

      turn.configSelectedFeedbackSensor(FeedbackDevice.CTRE_MagEncoder_Relative);
      turn.setSensorPhase(true);
      turn.config_kP(0, Constants.TURN_P);
      turn.config_kI(0, Constants.TURN_I);
      turn.config_kD(0, Constants.TURN_D);

      // drive.configPeakCurrentDuration(10);
      drive.configSupplyCurrentLimit(new SupplyCurrentLimitConfiguration(true, 15, 0, 0.01));
      // drive.enableCurrentLimit(true);ddp
  }

  /**
   * Sets a pod to a specified angle and velocity
   * 
   * @param pod: The pod to move
   * @param angle: Pod angle in degrees
   * @param velocity: Pod velocity in meters per second
   */
  public void set(double angle, double velocity) {
    // TODO: I can't remember if we waited for the angle target to be met before we
    // start to apply the drive velocity, look into that
    double drivePercent = velocity / Constants.DRIVE_MAX_SPEED;
    if(Math.abs(drivePercent) > 0.05)
      turn.set(ControlMode.Position, angle / 360.0 * Constants.POD_COUNTS_PER_REV);
    drive.set(ControlMode.PercentOutput, drivePercent);
  }

  public void zeroEncoder() {
    turn.setSelectedSensorPosition(0);
  }

  public double getAngle() {
    return turn.getSelectedSensorPosition() / Constants.POD_COUNTS_PER_REV * 360.0;
  }
}
