package frc.robot.commands;

import edu.wpi.first.math.geometry.Rotation2d;
import edu.wpi.first.math.kinematics.ChassisSpeeds;
import edu.wpi.first.wpilibj.Timer;
import edu.wpi.first.wpilibj2.command.CommandBase;
import frc.robot.subsystems.Chassis;
import frc.robot.subsystems.Gatherer;
import frc.robot.subsystems.PotatoCannon;

public class Auto extends CommandBase {
  private Chassis m_chassis;
  private PotatoCannon m_cannon;
  private Gatherer m_gatherer;

  private double m_startTime;

  public Auto(Chassis chassis, PotatoCannon cannon, Gatherer gatherer) {
    addRequirements(chassis);
    addRequirements(cannon);
    addRequirements(gatherer);
    m_chassis = chassis;
    m_cannon = cannon;
    m_gatherer = gatherer;
  }

  @Override
  public void initialize() {
    m_startTime = Timer.getFPGATimestamp();
  }

  @Override
  public void execute() {
    double curTime = Timer.getFPGATimestamp();
    double time = curTime - m_startTime;
    m_cannon.set(0.95);
    if(time > 7.0) {
      // System.out.println("Stop");
      m_chassis.driveFR(0.0, 0.0, 0.0);
      m_cannon.set(0);
      m_gatherer.stop();
    } else if(time > 4.0) {
      // System.out.println("Move");
      ChassisSpeeds speeds = ChassisSpeeds.fromFieldRelativeSpeeds(
        2.0,
        0.0,
        0.0,
        Rotation2d.fromDegrees(0.0));
      m_chassis.drive(speeds);
      // m_chassis.driveFR(2.0, 0.0, 0.0);
    } else if(time > 2.0) {
      // System.out.println("Shoot");
      m_gatherer.shoot(false);
    }
  }

  @Override
  public void end(boolean interrupted) {
  }

  @Override
  public boolean isFinished() {
    return false;
  }
}
