package frc.robot.subsystems;

import com.ctre.phoenix.motorcontrol.ControlMode;
import com.ctre.phoenix.motorcontrol.can.TalonSRX;

import edu.wpi.first.wpilibj.DigitalInput;
import edu.wpi.first.wpilibj2.command.SubsystemBase;
import frc.robot.Constants;

public class Gatherer extends SubsystemBase {
  private enum GatherState {
    IDLE,
    WAITING
  }

  private enum ManageState {
    EMPTY,
    ENTERING_LOW,
    TRANSFERRING,
    LEAVING_HIGH
  }

  private enum BallState {
    NO_BALLS,
    ENTERING_LOW,
    LOW,
    TRANSFERRING,
    HIGH,
    TWO_BALLS,
    SATURATED
  }

  private TalonSRX m_gather_low = new TalonSRX(Constants.GATHER_LOW);
  private TalonSRX m_gather_high_back = new TalonSRX(Constants.GATHER_HIGH_BACK);
  private TalonSRX m_gather_high_front = new TalonSRX(Constants.GATHER_HIGH_FRONT);

  private DigitalInput m_lowSensor = new DigitalInput(Constants.GATHER_LOW_DIO);
  private DigitalInput m_highSensor = new DigitalInput(Constants.GATHER_HIGH_DIO);

  private int m_ballsInTransport = 0;

  private boolean m_prevLowSensor = false;
  private boolean m_prevHighSensor = false;

  private boolean m_haveLowBall = false;
  private boolean m_haveHighBall = false;

  private BallState m_BallState = BallState.NO_BALLS;
  private GatherState m_state = GatherState.IDLE;
  private ManageState m_manageState = ManageState.EMPTY;

  public double shoot_speed_front = 1.0;
  public double shoot_speed_back = 0.5;

  public Gatherer() {
    m_gather_low.setInverted(false);
    m_gather_high_back.setInverted(true);
    m_gather_high_front.setInverted(false);
  }

  public boolean haveOneBall() {
    return !m_lowSensor.get();
  }

  public boolean haveTwoBall() {
    return false;
  }

  public void gather() {
    // m_state = GatherState.GATHERING;
    setGather(1);
    // setTower(0.6);
    m_gather_high_back.set(ControlMode.PercentOutput, -0.6);
    m_gather_high_front.set(ControlMode.PercentOutput, 0.6);
  }

  public void store() {
    setGather(1);
    m_gather_high_back.set(ControlMode.PercentOutput, shoot_speed_back);
    m_gather_high_front.set(ControlMode.PercentOutput, shoot_speed_front);
  }

  public void eject() {
    // m_state = GatherState.EJECTING;
    setGather(-1);
    setTower(-0.6);
  }

  public void shoot(boolean full) {
    // m_state = GatherState.SHOOTING;
    // setTower(1);

    if(full) {
      setGather(1.0);
    }
    
    m_gather_high_back.set(ControlMode.PercentOutput, shoot_speed_back);
    m_gather_high_front.set(ControlMode.PercentOutput, shoot_speed_front);
  }

  public void stop() {
    m_state = GatherState.IDLE;
    setGather(0);
    setTower(0);
  }

  public void setGather(double speed) {
    m_gather_low.set(ControlMode.PercentOutput, speed);
  }

  public void setTower(double speed) {
    m_gather_high_back.set(ControlMode.PercentOutput, speed);
    m_gather_high_front.set(ControlMode.PercentOutput, speed);
  }

  @Override
  public void periodic() {
    boolean lowSensor = m_lowSensor.get();
    boolean highSensor = m_highSensor.get();

    boolean lowStateChanged = false;
    boolean highStateChanged = false;

    if(m_prevLowSensor != lowSensor) {
      lowStateChanged = true;
      m_prevLowSensor = lowSensor;
    }
    if(m_prevHighSensor != highSensor) {
      highStateChanged = true;
      m_prevHighSensor = highSensor;
    }

    switch(m_BallState) {
      case NO_BALLS:
        break;
      case ENTERING_LOW:
        // setGather(1);
        break;
      case LOW:
        break;
      case TRANSFERRING:
        break;
      case HIGH:
        break;
      case TWO_BALLS:
        break;
      case SATURATED:
        break;
    }

    // switch(m_state) {
    //   case IDLE:
    //     setGather(0);
    //     setTower(0);
    //     break;
    //   case GATHERING:
    //     setGather(1);
    //     break;
    //   case EJECTING:
    //     setGather(-1);
    //     break;
    //   case SHOOTING:
    //     setTower(1);
    //     break;
    //   case MANAGING:
    //     manage();
    //     break;
    // }
  }

  private void manage() {

    switch(m_manageState) {
      case EMPTY:
        break;
      case ENTERING_LOW:
        setGather(1);
        // if(lowStateChanged && lowSensor) {
        //   setGather(0);
        //   m_haveLowBall = true;
        // }
        break;
      case TRANSFERRING:
        break;
      case LEAVING_HIGH:
        break;
    }
  }

  @Override
  public void simulationPeriodic() {
    // This method will be called once per scheduler run during simulation
  }
}
